local ADDON_NAME = ...

COMMENTATOR_FIX_TEAM_SIDES = true

BINDING_HEADER_COMMENTATORFOLLOW_BG = "BG — Following"
BINDING_HEADER_COMMENTATORFOLLOWSNAP_BG = "BG — Snapping"
BINDING_HEADER_COMMENTATORLOOKAT_BG = "BG — Tracking"

BINDING_NAME_COMMENTATORFOLLOW13 = "Follow Player 7 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW14 = "Follow Player 8 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW15 = "Follow Player 9 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW16 = "Follow Player 10 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW17 = "Follow Player 7 on Team 2"
BINDING_NAME_COMMENTATORFOLLOW18 = "Follow Player 8 on Team 2"
BINDING_NAME_COMMENTATORFOLLOW19 = "Follow Player 9 on Team 2"
BINDING_NAME_COMMENTATORFOLLOW20 = "Follow Player 10 on Team 2"

BINDING_NAME_COMMENTATORFOLLOW13SNAP = "Snap to Player 7 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW14SNAP = "Snap to Player 8 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW15SNAP = "Snap to Player 9 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW16SNAP = "Snap to Player 10 on Team 1"
BINDING_NAME_COMMENTATORFOLLOW17SNAP = "Snap to Player 7 on Team 2"
BINDING_NAME_COMMENTATORFOLLOW18SNAP = "Snap to Player 8 on Team 2"
BINDING_NAME_COMMENTATORFOLLOW19SNAP = "Snap to Player 9 on Team 2"
BINDING_NAME_COMMENTATORFOLLOW20SNAP = "Snap to Player 10 on Team 2"

BINDING_NAME_COMMENTATORLOOKAT13 = "Track Player 7 on Team 1"
BINDING_NAME_COMMENTATORLOOKAT14 = "Track Player 8 on Team 1"
BINDING_NAME_COMMENTATORLOOKAT15 = "Track Player 9 on Team 1"
BINDING_NAME_COMMENTATORLOOKAT16 = "Track Player 10 on Team 1"
BINDING_NAME_COMMENTATORLOOKAT17 = "Track Player 7 on Team 2"
BINDING_NAME_COMMENTATORLOOKAT18 = "Track Player 8 on Team 2"
BINDING_NAME_COMMENTATORLOOKAT19 = "Track Player 9 on Team 2"
BINDING_NAME_COMMENTATORLOOKAT20 = "Track Player 10 on Team 2"


local function SetWidgetContainerPosition()
    if not InCombatLockdown() then
        UIWidgetTopCenterContainerFrame:ClearAllPoints()
        UIWidgetTopCenterContainerFrame:SetPoint("TOP", Commentator.Scoreboard, "BOTTOM", 0, -45)
    end
end

local function SetUnitSpecialization(unitFrame)
    local inInstance, instanceType = IsInInstance()
    if instanceType == "pvp" then
        local playerData = unitFrame.playerData
        if not playerData and UnitIsPlayer(unitFrame.unitToken) then
            for teamIndex = 1, 2 do
                for _, frame in ipairs(Commentator.unitFrames[teamIndex]) do
                    if frame:GetGUID() == unitFrame:GetGUID() then
                        playerData = frame.playerData
                        break
                    end
                end
            end
        end
        if playerData and playerData.specialization then
            local _, _, _, icon = GetSpecializationInfoByID(playerData.specialization)
            unitFrame.Circle.ClassIcon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
            SetPortraitToTexture(unitFrame.Circle.ClassIcon, icon)
        end
    end
end

local function SetSpectatedUnitToken(unitFrame)
    local playerData = unitFrame.playerData
    local teamIndex = unitFrame.teamIndex

    local team = "a"
    if teamIndex == 1 then
        team = "b"
    end

    local numPlayers = C_Commentator.GetNumPlayers(teamIndex)
    for i = 1, numPlayers do
        local spectatedToken = "spectated" .. team .. i
        local name = GetUnitName(spectatedToken, true) or ""
        local overrideName = C_Commentator.GetPlayerOverrideName(name)
        local name = overrideName or UnitName(spectatedToken)
        if name == playerData.name then
            playerData.unitToken = spectatedToken
            local isAlignedLeft = teamIndex == 1
            unitFrame:Init(isAlignedLeft, playerData, teamIndex)
            return
        end
    end
end

local frame = CreateFrame("Frame")

frame:SetScript("OnEvent", function(self, event, ...)
    if event == "PLAYER_LOGIN" then
        if CommentatorTargetFrame then
            hooksecurefunc(CommentatorTargetFrame, "SetClass", SetUnitSpecialization)
            hooksecurefunc(CommentatorTargetOfTargetFrame, "SetClass", SetUnitSpecialization)
        end
    elseif event == "PLAYER_ENTERING_WORLD" then
        COMMENTATOR_FIX_TEAM_SIDES = true
    end
end)

frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

hooksecurefunc(Commentator, "SetFrameLock", function(self, enabled)
    local inInstance, instanceType = IsInInstance()
    if instanceType == "pvp" and enabled then
        SetWidgetContainerPosition()
        UIWidgetTopCenterContainerFrame:originalShow()
    end
end)

hooksecurefunc(Commentator, "InitUnitFrames", function(self)
    local inInstance, instanceType = IsInInstance()

    if instanceType ~= "pvp" then
        return
    end

    local originY = -25
    local padding = -130
    local offsetX = 20
    do
        local offsetY = originY
        for index, unitFrame in ipairs(self.unitFrames[1]) do
            if not InCombatLockdown() then
                unitFrame:ClearAllPoints()
                unitFrame:SetPoint("TOPLEFT", self, offsetX, offsetY)
                offsetY = offsetY + padding
                unitFrame:SetScale(0.75)
                unitFrame.DefensiveSpellTray:Hide()
                unitFrame.DebuffSpellTray:Hide()
                unitFrame.OffensiveSpellTray:Hide()
                SetSpectatedUnitToken(unitFrame)
                SetUnitSpecialization(unitFrame)
            end
        end
    end
    do
        local offsetY = originY
        for index, unitFrame in ipairs(self.unitFrames[2]) do
            if not InCombatLockdown() then
                unitFrame:ClearAllPoints()
                unitFrame:SetPoint("TOPRIGHT", self, -offsetX, offsetY)
                offsetY = offsetY + padding
                unitFrame:SetScale(0.75)
                unitFrame.DefensiveSpellTray:Hide()
                unitFrame.DebuffSpellTray:Hide()
                unitFrame.OffensiveSpellTray:Hide()
                SetSpectatedUnitToken(unitFrame)
                SetUnitSpecialization(unitFrame)
            end
        end
    end

    if COMMENTATOR_FIX_TEAM_SIDES and not InCombatLockdown() then
        local unitFrame = self.unitFrames[1][1]
        local unitFrame2 = self.unitFrames[2][1]
        if unitFrame and unitFrame2 then
            local faction = unitFrame.playerData.faction
            local teamIndex = unitFrame.teamIndex
            if (faction == 0 and teamIndex == 1) or (faction == 1 and teamIndex == 2) then
                print("Swapping team sides...")
                self:SwapTeams()
            end
            COMMENTATOR_FIX_TEAM_SIDES = false
        end
    end
end)

hooksecurefunc(Commentator, "OnUpdate", function(self, elapsed)
    local inInstance, instanceType = IsInInstance()

    if instanceType ~= "pvp" then
        return
    end

    for _, widgetFrame in pairs(UIWidgetTopCenterContainerFrame.widgetFrames) do
        if widgetFrame.widgetType == 3 then
            local _, relativeFrame = widgetFrame.LeftBar:GetPoint()
            widgetFrame.LeftBar:ClearAllPoints()
            widgetFrame.LeftBar:SetPoint("TOPRIGHT", relativeFrame, "TOP", -64, 42)
            widgetFrame.RightBar:ClearAllPoints()
            widgetFrame.RightBar:SetPoint("TOPLEFT", relativeFrame, "TOP", 64, 42)
        elseif widgetFrame.widgetType == 14 then
            local ofsx = 188
            local ofsy = 72
            if widgetFrame.widgetID == 1640 then
                ofsx = 184
                ofsy = 78
            elseif widgetFrame.widgetID == 1683 then
                ofsy = 76
            end
            for icon in widgetFrame.iconPool:EnumerateActive() do
                local point, relativeTo, relativePoint, xOfs, yOfs = icon:GetPoint()
                if relativeTo == icon:GetParent() then
                    if point == "TOPLEFT" then
                        icon:ClearAllPoints()
                        icon:SetPoint(point, relativeTo, relativePoint, ofsx, ofsy)
                    else
                        icon:ClearAllPoints()
                        icon:SetPoint(point, relativeTo, relativePoint, -ofsx, ofsy)
                    end
                end
            end
        elseif widgetFrame.widgetType == 0 or widgetFrame.widgetID == 6 then
            widgetFrame:AnimOut()
        end
    end
end)
